# signal handling for Python GTK

%{
/* signal marshallers -- Ideas Stolen from Perl Gtk*/
static PyObject *GtkArg_to_Tuple(int nparams, GtkArg *args) {
    PyObject *params;
    int i;
    char *name;

    params = PyTuple_New(nparams);
    for (i=0; i < nparams; i++)
        switch (GTK_FUNDAMENTAL_TYPE(args[i].type)) {
            case GTK_TYPE_INVALID:
                fprintf(stderr, "invalid argument type?\n");
                Py_INCREF(Py_None);
                PyTuple_SetItem(params, i, Py_None);
                break;
            case GTK_TYPE_NONE:
                Py_INCREF(Py_None);
                PyTuple_SetItem(params, i, Py_None);
                break;
            case GTK_TYPE_CHAR:
                PyTuple_SetItem(params, i, PyString_FromStringAndSize(
                      &(GTK_VALUE_CHAR(args[i])), 1));
                break;
            case GTK_TYPE_BOOL:
                PyTuple_SetItem(params, i, PyInt_FromLong(GTK_VALUE_BOOL(
                      args[i])));
                break;
            case GTK_TYPE_INT:
                PyTuple_SetItem(params, i, PyInt_FromLong(GTK_VALUE_INT(
                      args[i])));
                break;
            case GTK_TYPE_UINT:
                PyTuple_SetItem(params, i, PyInt_FromLong(GTK_VALUE_UINT(
                      args[i])));
                break;
            case GTK_TYPE_LONG:
                PyTuple_SetItem(params, i, PyInt_FromLong(GTK_VALUE_LONG(
                      args[i])));
                break;
            case GTK_TYPE_ULONG:
                PyTuple_SetItem(params, i, PyInt_FromLong(GTK_VALUE_ULONG(
                      args[i])));
                break;
            case GTK_TYPE_FLOAT:
                PyTuple_SetItem(params, i, PyFloat_FromDouble(
                      GTK_VALUE_FLOAT(args[i])));
                break;
            case GTK_TYPE_STRING:
                PyTuple_SetItem(params, i, PyString_FromString(
                      GTK_VALUE_STRING(args[i])));
                break;
            case GTK_TYPE_ENUM:
                PyTuple_SetItem(params, i, PyInt_FromLong(GTK_VALUE_ENUM(
                      args[i])));
                break;
            case GTK_TYPE_FLAGS:
                PyTuple_SetItem(params, i, PyInt_FromLong(GTK_VALUE_FLAGS(
                      args[i])));
                break;
            case GTK_TYPE_ARGS:
                PyTuple_SetItem(params, i, GtkArg_to_Tuple(GTK_VALUE_ARGS(
                           args[i]).n_args, GTK_VALUE_ARGS(args[i]).args));
                break;
            case GTK_TYPE_OBJECT:
                PyTuple_SetItem(params, i, PyGtk_New(GTK_VALUE_OBJECT(
                    args[i])));
                break;
            case GTK_TYPE_POINTER:
                PyTuple_SetItem(params, i,PyInt_FromLong((long)
                    GTK_VALUE_POINTER(args[i])));
                break;
            case GTK_TYPE_BOXED:
                name = gtk_type_name(args[i].type);
                /* conversion is specific to the type which is boxed
                 * builtin boxed types: GtkAcceleratorTable, GtkStyle,
                 *   GdkColormap, GdkVisual, GdkFont, GdkWindow, GdkEvent */
                break;
/*            case GTK_TYPE_FOREIGN:
            case GTK_TYPE_CALLBACK:
            case GTK_TYPE_SIGNAL:
*/            default:
                break;
        }
    return params;
}

static
void PyGtk_SignalMarshal(GtkObject *object, /*gpointer*/ PyObject *func,
                       int nparams, GtkArg *args, GtkType *arg_types,
                                                 GtkType return_type) {
    PyObject *arg_list, *params, *ret;
    int i;

    ret = PyTuple_New(1);
    PyTuple_SetItem(ret, 0, PyGtk_New(object));

    for (i=0; i < nparams; i++) {
        if (arg_types[i] != GTK_TYPE_NONE) {
            args[i].type = arg_types[i];
            args[i].name = NULL;
        }
    }
    arg_list = GtkArg_to_Tuple(nparams, args);
    params = PySequence_Concat(ret, arg_list);
    Py_DECREF(ret);
    Py_DECREF(arg_list);
    ret = PyObject_CallObject(func, params);
    Py_DECREF(params);
    if (ret == NULL) {
        PyErr_Clear();
        fprintf(stderr, "Error - Exception ignored\n");
        return;
    }
    Py_DECREF(ret);
}

static
void PyGtk_SignalDestroy(/*gpointer*/ PyObject *func) {
    Py_DECREF(func);
}

static PyObject *gtk__signal_connect(PyObject *self, PyObject *args) {
    PyGtk_Object *obj;
    char *name;
    PyObject *func;
    int signum;

    if (!PyArg_ParseTuple(args, "O!sO:gtk_signal_connect", &PyGtk_Type, &obj,
                    &name, &func))
        return NULL;
    if (!PyCallable_Check(func)) {
        PyErr_SetString(PyExc_TypeError, "third argument must be callable");
        return NULL;
    }
    Py_INCREF(func);
    signum = gtk_signal_connect(PyGtk_Get(obj), name, NULL, func);
    return PyInt_FromLong(signum);
}

static PyObject *gtk__signal_connect_after(PyObject *self, PyObject *args) {
    PyGtk_Object *obj;
    char *name;
    PyObject *func;
    int signum;

    if (!PyArg_ParseTuple(args, "O!sO:gtk_signal_connect_after",
                    &PyGtk_Type, &obj, &name, &func))
        return NULL;
    if (!PyCallable_Check(func)) {
        PyErr_SetString(PyExc_TypeError, "third argument must be callable");
        return NULL;
    }
    Py_INCREF(func);
    signum = gtk_signal_connect_after(PyGtk_Get(obj), name, NULL, func);
    return PyInt_FromLong(signum);
}

%}

// I haven't included gtk_signal_connect_object, since it is difficult
// to implement in C.  Here is a sample implementation in Python:
//
//  def gtk_signal_connect_object(obj, name, func, cobj):
//    class sigfunc:
//      def __init__(self, obj, func): self.obj=obj ; self.func = func
//      def __call__(self, obj, *args): apply(self.func, (self.obj,)+args)
//    return gtk_signal_connect(obj, name, sigfunc(cobj, func))
//
%native(gtk_signal_connect) PyObject *gtk__signal_connect(
        PyObject *self, PyObject *args);
%native(gtk_signal_connect_after) PyObject *gtk__signal_connect_after(
        PyObject *self, PyObject *args);

void gtk_signal_disconnect(GtkObject *o, int signum);
void gtk_signal_disconnect_by_data(GtkObject *o, PyObject *func);
void gtk_signal_handler_block(GtkObject *o, int signum);
void gtk_signal_handler_block_by_data(GtkObject *o, PyObject *func);
void gtk_signal_handler_unblock(GtkObject *o, int signum);
void gtk_signal_handler_unblock_by_data(GtkObject *o, PyObject *func);
void gtk_signal_handlers_destroy(GtkObject *o);

